/************************************************************************/
/*                                                                      */
/*								Utilities								*/
/*                                                                      */
/************************************************************************/

#include "cpu.h"

static int NextSystemLabel = 1000;		/* ID next system label */
static BOOL Defined[512] = { FALSE };	/* TRUE if opcode defined */
static int Count = 0;					/* No of instructions emulated */
static BOOL InDef = FALSE;
static int Current = -1;

/************************************************************************/
/*                                                                      */
/*					Allocate labels for code generation					*/
/*                                                                      */
/************************************************************************/

int	ClaimLabel(int Required)
{
int n;
ASSERT(Required > 0 && Required < 10);
ASSERT(NextSystemLabel < 32000);
n = NextSystemLabel;
NextSystemLabel = NextSystemLabel+Required;
return(n);
}

/************************************************************************/
/*                                                                      */
/*						Define a new opcode								*/
/*                                                                      */
/************************************************************************/

void Define(int Opcode,int Cycles,char *Command)
{
char s1[80];

ASSERT(Opcode >= 0 && Opcode < 512);
ASSERT(Cycles >= 0 && Cycles < 200);
ASSERT(ISSTRING(Command));
ASSERT(!Defined[Opcode]);
ASSERT(!InDef);

InDef = TRUE;
Defined[Opcode] = TRUE;
sprintf(s1,"%s [%s%02x] %d",
		Command,(Opcode > 255) ? "CB" : "",Opcode & 255,Cycles);
StarBar(s1);
Label(OpLabel(Opcode));
if (Cycles != 0)
	printf("    mov [%s],%d\n",GetZRegName("cycles"),Cycles);
Current = Opcode;
}

/************************************************************************/
/*                                                                      */
/*								Draw a starbar							*/
/*                                                                      */
/************************************************************************/

void StarBar(char *Title)
{
char s1[80],s2[80];
ASSERT(ISSTRING(Title));
ASSERT(strlen(Title) < 60);
strcpy(s1,Title);
while (strlen(s1) < 60)
	{
	sprintf(s2," %s ",s1);strcpy(s1,s2);
	}
sprintf(s2,"***** %s *****",s1);
*s1 = '\0';
while (strlen(s1) < strlen(s2)) strcat(s1,"*");
printf(";%s\n;%s\n;%s\n",s1,s2,s1);
}

/************************************************************************/
/*                                                                      */
/*						End of a definition								*/
/*                                                                      */
/************************************************************************/

void EndDefine(void)
{
ASSERT(InDef);
InDef = FALSE;
printf("    jmp edi ; end of %d \n",++Count);
}

/************************************************************************/
/*                                                                      */
/*						Exit with a fatal error							*/
/*                                                                      */
/************************************************************************/

void Fatal(int Line,char *File)
{
fprintf(stderr,"Assert failed : %d of %s\n",Line,File);
exit(0);
}

/************************************************************************/
/*                                                                      */
/*	   Generate code to load register A,B,C,D,E,H,L,(HL) into Reg86		*/
/*                                                                      */
/************************************************************************/

void GetReg(int Number,char *Reg86)
{
ASSERT(IS8086REG(Reg86));
ASSERT(IS8BITREG(Number));
if (Number == REG_HL)
	{
	printf("    mov bx,[word ptr %s]\n",GetZRegName("HL"));
	MemRead(Reg86);
	}
else
	{
	printf("    mov %s,[%s]\n",Reg86,GetZRegName(GetRegName(Number)));
	}
}

/************************************************************************/
/*                                                                      */
/*			Generate code to put 8086 Register into Z80 Register		*/
/*                                                                      */
/************************************************************************/

void PutReg(int Number,char *Reg86)
{
ASSERT(IS8086REG(Reg86));
ASSERT(IS8BITREG(Number));
if (Number == REG_HL)
	{
	printf("    mov bx,[word ptr %s]\n",GetZRegName("HL"));
	MemWrite(Reg86);
	}
else
	{
	printf("    mov [%s],%s\n",GetZRegName(GetRegName(Number)),Reg86);
	}
}

/************************************************************************/
/*                                                                      */
/*						Get name of z80 register						*/
/*                                                                      */
/************************************************************************/

char *GetRegName(int Number)
{
ASSERT(IS8BITREG(Number));
if (Number == REG_HL) return("(hl)");
return("b\0c\0d\0e\0h\0l\0?\0a"+Number*2);
}

/************************************************************************/
/*                                                                      */
/*					Get Identifier for given register					*/
/*                                                                      */
/************************************************************************/

static char GZRBuffer[16];

char *GetZRegName(char *Name)
{
ASSERT(ISZ80REG(Name));
strcpy(GZRBuffer,"ZREG");
strcat(GZRBuffer,Name);
strlwr(GZRBuffer);
return(GZRBuffer);
}

/************************************************************************/
/*                                                                      */
/*							Define a label								*/
/*                                                                      */
/************************************************************************/

void Label(char *Lbl)
{
printf("%s:\n",Lbl);
}

/************************************************************************/
/*                                                                      */
/*					Create opcode label from opcode						*/
/*                                                                      */
/************************************************************************/

static char OLBuffer[32];

char *OpLabel(int Opcode)
{
ASSERT(Opcode >= 0 && Opcode < 512);
sprintf(OLBuffer,"opc00%02X",Opcode & 255);
if (Opcode > 255)
	{
	OLBuffer[3] = 'c';OLBuffer[4] = 'b';
	}
return(OLBuffer);
}

/************************************************************************/
/*                                                                      */
/*					Create system label from number						*/
/*                                                                      */
/************************************************************************/

static char SLBuffer[32];

char *SysLabel(int Label)
{
ASSERT(Label >= 1000 && Label < NextSystemLabel);
sprintf(SLBuffer,"SYSL%05d",Label);
return(SLBuffer);
}

/************************************************************************/
/*                                                                      */
/*					Compile code to set N flag							*/
/*                                                                      */
/************************************************************************/

void SetNFlag(BOOL Set)
{
ASSERT(ISBOOL(Set));
printf("    mov [%s],%d\n",GetZRegName("NFlag"),Set ? 255 : 0);
}

/************************************************************************/
/*                                                                      */
/*		Search for string in string list. Return position or -1			*/
/*                                                                      */
/************************************************************************/

int InString(char *s1,char *s2)
{
int n = 0;
ASSERT(ISSTRING(s1));
ASSERT(ISSTRING(s2));
while (stricmp(s1,s2) != 0)
	{
	if (*s1 == '\0') return(-1);
	s1 = s1 + strlen(s1) + 1;
	n++;
	}
return(n);
}

/************************************************************************/
/*                                                                      */
/*						Generate the vector tables						*/
/*                                                                      */
/************************************************************************/

void GenerateVectorTables(void)
{
int i;
printf(".data\nalign dword\n");
for (i = 0;i < 512;i++)
	{
	if (i == 0) Label("OPV00");
	if (i == 256) Label("OPVCB");
	if (i % 8 == 0) printf("    dd ");
	if (Defined[i]) printf("%s",OpLabel(i));
	else			printf("%s","GoDOS");
	if (i % 8 == 7) printf("\n"); else printf(",");
	}
printf(".code\n");
}

